package com.unissoft.basic.service.impl;

import com.alibaba.fastjson2.JSON;
import com.alibaba.fastjson2.JSONArray;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.unissoft.basic.model.BasicTimingTask;
import com.unissoft.basic.mapper.BasicTimingTaskMapper;
import com.unissoft.basic.service.BasicTimingTaskService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.unissoft.common.CronTaskRegistrar;
import com.unissoft.common.DateUtils;
import com.unissoft.common.PageParam;
import com.unissoft.common.SchedulingRunnable;
import com.unissoft.enums.BeanEnum;
import com.unissoft.enums.SysJobStatus;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.StringUtils;

import javax.annotation.Resource;
import java.util.*;

/**
 * <p>
 * 定时任务设置表 服务实现类
 * </p>
 *
 * @author root
 * @since 2023-02-06
 */
@Service
public class BasicTimingTaskServiceImpl extends ServiceImpl<BasicTimingTaskMapper, BasicTimingTask> implements BasicTimingTaskService {
    @Resource
    private BasicTimingTaskMapper timingTaskMapper;
    @Resource
    private CronTaskRegistrar cronTaskRegistrar;

    @Override
    @Transactional(rollbackFor = {Exception.class})
    public void saveList(BasicTimingTask timingTask) {
        // 保存
        if (!Objects.isNull(timingTask)) {
            timingTask.setId(UUID.randomUUID().toString());
            timingTask.setBuildTime(DateUtils.getDate());
        }
        // 设备任务信息处理
        this.joiningString(timingTask);
        int insert = timingTaskMapper.insert(timingTask);
        // 新增定时任务
        if (insert > 0) {
            if (timingTask.getOnOrOff().equals(SysJobStatus.NORMAL.ordinal())) {
                // beanName: 执行逻辑类,methodName类方法,param: 设备编码+通讯参数(待定)
                SchedulingRunnable task = new SchedulingRunnable(
                        // 设置线程内容
                        timingTask.getBeanName(), timingTask.getMethodName(), timingTask.getMethodParams()
                );
                cronTaskRegistrar.addCronTask(task, timingTask.getDetectionCron());
            }
        }
    }

    @Override
    @Transactional(rollbackFor = {Exception.class})
    public void updateList(BasicTimingTask timingTask) {
        /* 数据处理 */
        joiningString(timingTask);
        int i = timingTaskMapper.updateById(timingTask);
        // 修改定时任务,先移除原来的任务,再启动新任务
        if (i > 0) {
            // 先移除
            SchedulingRunnable task = new SchedulingRunnable(timingTask.getBeanName(), timingTask.getMethodName(), timingTask.getMethodParams());
            cronTaskRegistrar.removeCronTask(task);
            // 再添加
            if (timingTask.getOnOrOff().equals(SysJobStatus.NORMAL.ordinal())) {
                SchedulingRunnable taskAdd = new SchedulingRunnable(timingTask.getBeanName(), timingTask.getMethodName(), timingTask.getMethodParams());
                cronTaskRegistrar.addCronTask(taskAdd, timingTask.getDetectionCron());
            }
        }
    }

    @Override
    public IPage<BasicTimingTask> getPageList(PageParam pageParam) {
        // 创建分页条件对象
        IPage<BasicTimingTask> page = new Page<>(pageParam.getPageIndex(), pageParam.getPageSize());
        // 查询对象
        QueryWrapper<BasicTimingTask> qw = new QueryWrapper<>();
        // 解析查询条件
        String condition = pageParam.getCondition();
        if (!StringUtils.isEmpty(condition)) {
            BasicTimingTask timingTask = JSON.parseObject(condition, BasicTimingTask.class);
            // 查询条件
            String orgCode = timingTask.getOrgCode();
            String deviceType = timingTask.getDeviceType();
            Integer onOrOff = timingTask.getOnOrOff();
            String taskName = timingTask.getTaskName();
            if (!StringUtils.isEmpty(orgCode))
                qw.eq("org_code", orgCode);
            if (!StringUtils.isEmpty(deviceType))
                qw.eq("device_type", deviceType);
            if (Objects.nonNull(onOrOff))
                qw.eq("on_or_off", onOrOff);
            if (!StringUtils.isEmpty(taskName))
                qw.like("task_name", taskName);
        }
        qw.orderByDesc("build_time");
        return timingTaskMapper.getPageList(page, qw);
    }

    @Override
    @Transactional(rollbackFor = {Exception.class})
    public boolean deleteById(BasicTimingTask timingTask) {
        String beanName = timingTask.getBeanName();
        String methodName = timingTask.getMethodName();
        String methodParams = timingTask.getMethodParams();
        if (StringUtils.isEmpty(beanName)
                || StringUtils.isEmpty(methodName)
                || StringUtils.isEmpty(methodParams)) {
            return false;
        }
        if (timingTaskMapper.deleteById(timingTask.getId()) > 0) {
            // 移除
            SchedulingRunnable task = new SchedulingRunnable(timingTask.getBeanName(), timingTask.getMethodName(), timingTask.getMethodParams());
            cronTaskRegistrar.removeCronTask(task);
            return true;
        }
        return false;
    }

    /**
     * 参数设置
     *
     * @param timingTask 填充对象数据
     */
    private synchronized void joiningString(BasicTimingTask timingTask) {
        // 编码转换bean
        String pcc = timingTask.getDeviceType();
        BeanEnum beanEnum = BeanEnum.convertMethod(pcc);
        // 获取beanName
        timingTask.setBeanName(beanEnum.getBeanName());
        // 获取methodName
        timingTask.setMethodName(beanEnum.getMethodName());
        String methodParams = timingTask.getMethodParams();
        if (!StringUtils.isEmpty(methodParams)) {
            JSONArray objects = JSONArray.parseArray(methodParams);
            // 设备数量
            timingTask.setEqSum(objects.size());
        }
    }

    @Override
    public List<Map<String, Object>> getByDeviceName(String classifyName) {
        // 查询设备分类,获取所有子类分类标识
        List<String> list = timingTaskMapper.getByDeviceName(classifyName);
        // 如果不为空,查询设备配置信息
        if (list.size() > 0) {
            // 粮情检测设备 basic_device_config_food
            List<Map<String, Object>> configFood = timingTaskMapper.getConfigFood(list);
            if (configFood.size() <= 0) {
                // 气调设备 basic_device_config_gas
                List<Map<String, Object>> configGas = timingTaskMapper.getConfigGas(list);
                if (configGas.size() <= 0) {
                    // TODO 之后用哪个设备,加哪个判断
                    // 氮气设备 basic_device_config_nitrogen
                    List<Map<String, Object>> configNitrogen = timingTaskMapper.getConfigNitrogen(list);
                    if (configNitrogen.size() <= 0) {
                        // 气体检测设备 basic_device_config_atmosphere
                        List<Map<String, Object>> configAtmosphere = timingTaskMapper.getConfigAtmosphere(list);
                        if (configAtmosphere.size() <= 0) {
                            // 虫情设备 basic_device_config_pestis
                            List<Map<String, Object>> configPestis = timingTaskMapper.getConfigPestis(list);
                            if (configPestis.size() <= 0) {
                                // 能耗设备 basic_device_config_energy
                                List<Map<String, Object>> configEnergy = timingTaskMapper.getConfigEnergy(list);
                                if (configEnergy.size() <= 0) {
                                    // 气象站设备 basic_device_config_meteorological
                                    List<Map<String, Object>> configMeteorological = timingTaskMapper.getConfigMeteorological(list);
                                    if (configMeteorological.size() <= 0) {
                                        // 气密性设备 basic_device_config_airtightness
                                        List<Map<String, Object>> configAirtightness = timingTaskMapper.getConfigAirtightness(list);
                                        if (configAirtightness.size() <= 0) {
                                            // 通风设备 basic_device_config_wind
                                            List<Map<String, Object>> configWind = timingTaskMapper.getConfigWind(list);
                                            if (configWind.size() <= 0) {
                                                // 照明设备 basic_device_config_lighting
                                                List<Map<String, Object>> configLighting = timingTaskMapper.getConfigLighting(list);
                                                if (configLighting.size() <= 0) {
                                                    // 控温设备 basic_device_config_temperature
                                                    List<Map<String, Object>> configTemperature = timingTaskMapper.getConfigTemperature(list);
                                                    if (configTemperature.size() <= 0) {
                                                        // end return empty
                                                        return Collections.emptyList();
                                                    } else {
                                                        return configTemperature;
                                                    }
                                                } else {
                                                    return configLighting;
                                                }

                                            } else {
                                                return configWind;
                                            }

                                        } else {
                                            return configAirtightness;
                                        }

                                    } else {
                                        return configMeteorological;
                                    }

                                } else {
                                    return configEnergy;
                                }

                            } else {
                                return configPestis;
                            }

                        } else {
                            return configAtmosphere;
                        }

                    } else {
                        return configNitrogen;
                    }
                } else {
                    return configGas;
                }
            } else {
                return configFood;
            }
        }
        // 都没有返回空集合
        return Collections.emptyList();
    }

}
