package com.unissoft.common;

import com.alibaba.fastjson2.JSON;
import com.unissoft.sys.model.SysLog;
import com.unissoft.sys.service.SysLogService;
import com.unissoft.user.model.UserInfo;
import lombok.extern.slf4j.Slf4j;
import org.aspectj.lang.JoinPoint;
import org.aspectj.lang.annotation.AfterReturning;
import org.aspectj.lang.annotation.Aspect;
import org.aspectj.lang.annotation.Pointcut;
import org.aspectj.lang.reflect.MethodSignature;
import org.springframework.stereotype.Component;
import org.springframework.web.context.request.RequestAttributes;
import org.springframework.web.context.request.RequestContextHolder;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.lang.reflect.Method;
import java.util.UUID;

/**
 * @author qingsong.han
 * @description: 操作日志拦截器
 * @create 2021-10-21 15:00
 */
@Slf4j
@Aspect
@Component
@SuppressWarnings("all")
public class LogAspect {

    @Resource
    private SysLogService logService;

    /*
     * 设置操作日志切入点 记录操作日志 在注解的位置切入代码
     */
    @Pointcut("@annotation(com.unissoft.common.SystemLog)")
    public void pointcut() {
    }

    /**
     * 正常返回通知，拦截用户操作日志，连接点正常执行完成后执行， 如果连接点抛出异常，则不会执行
     *
     * @param joinPoint 切入点
     * @param keys      返回结果
     */
    @AfterReturning(value = "pointcut()", returning = "keys")
    public void saveOperationLog(JoinPoint joinPoint, Object keys) {
        // 获取RequestAttributes
        RequestAttributes requestAttributes = RequestContextHolder.getRequestAttributes();
        // 从获取RequestAttributes中获取HttpServletRequest的信息
        HttpServletRequest request = (HttpServletRequest) requestAttributes
                .resolveReference(RequestAttributes.REFERENCE_REQUEST);

        SysLog sysLog = new SysLog();
        try {
            sysLog.setId(UUID.randomUUID().toString()); // 主键ID

            // 从切面织入点处通过反射机制获取织入点处的方法
            MethodSignature signature = (MethodSignature) joinPoint.getSignature();
            // 获取切入点所在的方法
            Method method = signature.getMethod();
            // 获取操作
            SystemLog opLog = method.getAnnotation(SystemLog.class);
            if (opLog != null) {
                String operationType = opLog.value(); // 获取注解参数
                sysLog.setOperationType(operationType); // 操作类型
            }
            UserInfo userInfo = (UserInfo) request.getSession().getAttribute("userInfo");
            if (userInfo != null) {
                sysLog.setUserName(userInfo.getUserName()); // 请求用户ID
                sysLog.setOrgCode(userInfo.getOrgCode()); // 请求用户组织机构编码
            } else {
                sysLog.setUserName("admin"); // 请求用户ID
                sysLog.setOrgCode("000"); // 请求用户组织机构编码
            }
            sysLog.setOperationTime(DateUtils.getDate()); // 创建时间
            logService.save(sysLog);
            log.info("logAspect返回结果: {}", JSON.toJSONString(keys)); // 返回结果
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

}
