package com.unissoft.common;

import com.fasterxml.jackson.annotation.JsonIgnore;
import com.fasterxml.jackson.annotation.JsonInclude;
import com.fasterxml.jackson.databind.JsonNode;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.databind.annotation.JsonSerialize;
import lombok.NoArgsConstructor;

import java.io.Serializable;
import java.util.List;

/**
 * @description 返回码工具类
 */
// 确保序列化JSON时，如果是null对象，其key也会消失。
@JsonSerialize(include = JsonSerialize.Inclusion.NON_NULL)
// 生成无参构造，确保在RPC调用时，不会出现反序列失败
@NoArgsConstructor
@JsonInclude(JsonInclude.Include.NON_NULL)
@SuppressWarnings("unused")
public class R<T> implements Serializable {

    // 定义jackson对象
    private static final ObjectMapper MAPPER = new ObjectMapper();

    /**
     * 状态码:200:成功
     */
    private int code;
    
    /**
     * 描述:notes
     */
    private String message;
    
    /**
     * 响应数据
     */
    private T data;

    private R(int code){
        this.code = code;
    }
    private R(int code, String message){
        this.code = code;
        this.message =message;
    }
    private R(int code, T data){
        this.code = code;
        this.data = data;
    }
    private R(int code, String message, T data ){
        this.code = code;
        this.message = message;
        this.data = data;
    }

    public int getcode() {
        return code;
    }

    public String getMessage() {
        return message;
    }

    public T getData() {
        return data;
    }

    @JsonIgnore
    public boolean isSuccess(){
       return this.code == ResultStatusEnum.SUCCESS.getCode();
    }

    public static<T> R<T> SUCCESS(){
        return new R<>(ResultStatusEnum.SUCCESS.getCode(), ResultStatusEnum.SUCCESS.getProductDescription());
    }
    public static<T> R<T> SUCCESS(String message){
        return new R<>(ResultStatusEnum.SUCCESS.getCode(), message);
    }
    public static<T> R<T> SUCCESS(T data){
        return new R<>(ResultStatusEnum.SUCCESS.getCode(), ResultStatusEnum.SUCCESS.getProductDescription(), data);
    }
    public static<T> R<T> SUCCESS(String message, T data){
        return new R<>(ResultStatusEnum.SUCCESS.getCode(), message, data);
    }
    public static<T> R<T> SUCCESS(int code, String message, T data){
        return new R<>(code, message, data);
    }

    public static<T> R<T> ERROR(){
        return new R<>(ResultStatusEnum.ERROR.getCode(), ResultStatusEnum.ERROR.getProductDescription());
    }
    public static<T> R<T> ERROR(String message){
        return new R<>(ResultStatusEnum.ERROR.getCode(), message);
    }
    public static<T> R<T> ERROR(String message, T data){
        return new R<>(ResultStatusEnum.ERROR.getCode(), message, data);
    }
    public static<T> R<T> ERROR(int code, String message){
        return new R<>(code, message);
    }
    public static<T> R<T> ERROR(int code, String message, T data){
        return new R<>(code, message, data);
    }

    /**
     * 将json结果集转化为ServerResponse对象
     *
     * @param jsonData json数据
     * @param clazz ServerResponse中的object类型
     * @return ServerResponse
     */
    public static R<?> formatToPojo(String jsonData, Class<?> clazz) {
        try {
            if (clazz == null) {
                return MAPPER.readValue(jsonData, R.class);
            }
            JsonNode jsonNode = MAPPER.readTree(jsonData);
            JsonNode data = jsonNode.get("data");
            Object obj = null;
            if (data.isObject()) {
                obj = MAPPER.readValue(data.traverse(), clazz);
            } else if (data.isTextual()) {
                obj = MAPPER.readValue(data.asText(), clazz);
            }
            return new R<>(jsonNode.get("code").intValue(), jsonNode.get("message").asText(), obj);
        } catch (Exception e) {
            return null;
        }
    }

    /**
     * 没有object对象的转化
     *
     * @param json String
     * @return ServerResponse
     */
    public static R<?> format(String json) {
        try {
            return MAPPER.readValue(json, R.class);
        } catch (Exception e) {
            e.printStackTrace();
        }
        return null;
    }

    /**
     * Object是集合转化
     *
     * @param jsonData json数据
     * @param clazz 集合中的类型
     * @return ServerResponse
     */
    public static R<?> formatToList(String jsonData, Class<?> clazz) {
        try {
            JsonNode jsonNode = MAPPER.readTree(jsonData);
            JsonNode data = jsonNode.get("data");
            Object obj = null;
            if (data.isArray() && data.size() > 0) {
                obj = MAPPER.readValue(data.traverse(),
                        MAPPER.getTypeFactory().constructCollectionType(List.class, clazz));
            }
            return new R<>(jsonNode.get("code").intValue(), jsonNode.get("message").asText(), obj);
        } catch (Exception e) {
            return null;
        }
    }


}
